﻿// Based on the Netlogo fire model
// http://ccl.northwestern.edu/netlogo/models/Fire

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace FireModel
{
    public partial class Form1 : Form
    {
        private Random rnd = new Random();
        // Forest dimensions
        private const int WIDTH = 150;
        private const int HEIGHT = 150;
        // 2D array to hold the forest
        private int[,] landscape = new int[WIDTH, HEIGHT];
        // Values in the forest 2D array
        // Distinguish between new and old fire
        private const int TREE = 1; 
        private const int CLEARING = 0;
        private const int FIRE = 2;
        private const int NEWFIRE = 3;
        
        public Form1()
        {
            InitializeComponent();
            // Set the width and height of the picturebox to 4 times the
            // dimensions of the forest
            pboxForest.Width = WIDTH * 4;
            pboxForest.Height = HEIGHT * 4;
        }

        // Randomly make each cell in the forest a tree or
        // a clearing based on the specified density
        private void btnSetup_Click(object sender, EventArgs e)
        {
            double probability = double.Parse(txtDensity.Text);

            // 0,0 is the upper left corner
            for (int x = 0; x < WIDTH; x++)
                for (int y = 0; y < HEIGHT; y++)
                {
                    if (rnd.NextDouble() < probability)
                        landscape[x, y] = TREE;
                    else
                        landscape[x, y] = CLEARING;
                }
            // Set all the trees on fire on the left side, where x = 0
            for (int y = 0; y < HEIGHT; y++)
                if (landscape[0,y] == TREE)
                    landscape[0, y] = FIRE;
            // Repaint the picturebox
            pboxForest.Invalidate();
        }

        // Draw the 2D array in the picturebox using a 4x4 pixel square
        // for each cell in the forest
        private void pboxForest_Paint(object sender, PaintEventArgs e)
        {
            // Draw the array as an image
            Graphics g = e.Graphics;
            Brush b;

            for (int x = 0; x < WIDTH; x++)
                for (int y = 0; y < HEIGHT; y++)
                {
                    if (landscape[x, y] == CLEARING)
                        b = Brushes.Black;
                    else if (landscape[x, y] == TREE)
                        b = Brushes.Green;
                    else
                        b = Brushes.Red;
                    g.FillRectangle(b, x*4, y*4, 4, 4);
                }
        }

        // Start a timer that burns the forest; we use a timer
        // here so we can see the forest burn step by step
        private void btnBurn_Click(object sender, EventArgs e)
        {
            timerBurn.Enabled = true;
        }      

        // This method ignites neighboring trees.
        // Returns false if nothing left to burn, true if something is still burning
        // It sets new burn to NEWFIRE so we can distinguish between old fire, then
        // it sets all NEWFIRE's to FIRE and all old FIRE to CLEARING
        private bool burnOneStep()
        {
            bool foundBurn = false;

            for (int x = 0; x < WIDTH; x++)
                for (int y = 0; y < HEIGHT; y++)
                {
                    if (landscape[x, y] == FIRE)
                    {
                        foundBurn = true;   // We found something burning

                        // Burn neighbor trees but don't go off the edges of the array
                        // Check left
                        if (x > 0)
                        {
                            if (landscape[x - 1, y] == TREE)
                                landscape[x - 1, y] = NEWFIRE;
                        }
                        // Check right
                        if (x < WIDTH - 1)
                        {
                            if (landscape[x + 1, y] == TREE)
                                landscape[x + 1, y] = NEWFIRE;
                        }
                        // Check up
                        if (y > 0)
                        {
                            if (landscape[x, y - 1] == TREE)
                                landscape[x, y - 1] = NEWFIRE;
                        }
                        // Check down
                        if (y < HEIGHT - 1)
                        {
                            if (landscape[x, y + 1] == TREE)
                                landscape[x, y + 1] = NEWFIRE;
                        }
                    }
                }
            if (foundBurn == false)
                return false;

            // Change all NEWFIRE's to FIRE and all FIRE to CLEARING
            for (int x = 0; x < WIDTH; x++)
                for (int y = 0; y < HEIGHT; y++)
                {
                    if (landscape[x, y] == FIRE)
                        landscape[x, y] = CLEARING;
                    else if (landscape[x, y] == NEWFIRE)
                        landscape[x, y] = FIRE;
                }
            return true;
        }

        // This method is called every 20 milliseconds (or whatever
        // value is set on the Timer).  Each time it is called we
        // burn neighboring trees.
        private void timer1_Tick(object sender, EventArgs e)
        {
            bool isBurning = burnOneStep();
            pboxForest.Invalidate();
            if (isBurning == false)
                timerBurn.Enabled = false;
        }
    }
}
